"""PermissionsX - Authorization for Django.

:copyright: Copyright (c) 2013-2017 by Robert Pogorzelski.
:license:   BSD, see LICENSE for more details.

"""
import importlib

from django.contrib import auth, messages
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django.views.generic import (
    RedirectView as DjangoRedirectView,
    View,
)

from permissionsx import settings
from permissionsx import base


class RedirectView(DjangoRedirectView):
    """View used when redirecting user with insufficient permissions."""

    permanent = False
    redirect_url = None

    def get_redirect_url(self, **kwargs):
        if self.redirect_url is None:
            self.redirect_url = reverse(settings.REDIRECT_URL_NAME)
        if self.request.user.is_authenticated():
            return self.redirect_url
        else:
            # NOTE: If user is not authenticated, return `redirect_url` with the `next` parameter.
            return self.redirect_url + '?next=' + self.request.get_full_path()


class MessageRedirectView(RedirectView):
    """Displays message after redirect.

    Inherits from :class:`RedirectView`. Usage:
    ::
        class AccessDeniedView(MessageRedirectView):

            message = (messages.warning, 'Access Denied')

    """
    message = (None, None)

    def get_message(self, request):
        return self.message

    def get(self, request, *args, **kwargs):
        msg_func, msg = self.get_message(request)
        if msg is not None:
            msg_func(request, msg)
        return super().get(request, *args, **kwargs)


class AccessDeniedView(MessageRedirectView):

    message = (messages.warning, _('You are not allowed to access this page!'))


class DjangoViewMixin(base.PermissionsCheckMixin):
    """Mixin required by any Django view used with permissions.

    :attr permissions: must be a function accepting `context` positional argument.
    :attr permissions_response_class: must be a subclass of :class:`View`.

    """
    permissions = base.default_permissions
    permissions_response_class = RedirectView

    def dispatch(self, request, *args, **kwargs):
        if self.permissions_check({'view': self, 'request': request}):
            return super().dispatch(request, *args, **kwargs)
        else:
            # NOTE: Unauthorized, logout if required and redirect by default.
            if settings.LOGOUT_IF_DENIED:
                auth.logout(request)
            return self.permissions_response_class.as_view()(request, *args, **kwargs)


# NOTE: A bit of magic happens here. The kind-of-factory below produces
#       Django views with `DjangoViewMixin` already mixed in, so it is
#       possible to:
#       `from permissionsx.contrib.django.views import PermissionsTemplateView`
#
generic_module = importlib.import_module('django.views.generic')
for key in dir(generic_module):
    obj = getattr(generic_module, key)
    try:
        if issubclass(obj, View):
            new_view = type('Permissions' + key, (DjangoViewMixin, obj), {})
            globals()[new_view.__name__] = new_view
            del new_view
    except TypeError:
        pass
