"""PermissionsX - Authorization for Django.

:copyright: Copyright (c) 2013-2017 by Robert Pogorzelski.
:license:   BSD, see LICENSE for more details.

"""
import json

from unittest import mock

from django.contrib import auth
from django.core.urlresolvers import reverse
from django.test.utils import override_settings

from permissionsx.base import Context
from permissionsx.contrib.django.views import RedirectView
from permissionsx.contrib.django.templatetags import permissions as permissions_tag
from permissionsx.tests import permissions
from permissionsx.tests.models import TestObject
from permissionsx.tests.utils import UtilityTestCase


class PermissionsDefinitionsTestCase(UtilityTestCase):

    def test_is_authenticated(self):
        request = self.get_request()
        context = Context({'request': request})
        self.assertFalse(
            permissions.is_authenticated(context)
        )
        request.user = self.user
        self.assertTrue(
            permissions.is_authenticated(context)
        )

    def test_is_superuser(self):
        request = self.get_request()
        self.assertFalse(
            permissions.is_authenticated(Context({'request': request}))
        )
        request.user = self.admin
        self.assertTrue(
            permissions.is_authenticated(Context({'request': request}))
        )
        self.assertTrue(
            permissions.is_superuser(Context({'request': request}))
        )

    def test_is_staff(self):
        request_staff = self.get_request(user=self.staff)
        request_admin = self.get_request(user=self.admin)
        self.assertTrue(
            permissions.is_staff(Context({'request': request_staff}))
        )
        self.assertFalse(
            permissions.is_staff(Context({'request': request_admin}))
        )

    def test_staff_or_superuser(self):
        request_staff = self.get_request(user=self.staff)
        request_admin = self.get_request(user=self.admin)
        self.assertTrue(
            permissions.is_staff_or_superuser(Context({'request': request_staff}))
        )
        self.assertTrue(
            permissions.is_staff_or_superuser(Context({'request': request_admin}))
        )

    def test_staff_and_superuser(self):
        request_staff = self.get_request(user=self.staff)
        request_admin = self.get_request(user=self.admin)
        request_user = self.get_request(user=self.user)
        self.assertFalse(
            permissions.is_staff_and_superuser(Context({'request': request_staff}))
        )
        self.assertFalse(
            permissions.is_staff_and_superuser(Context({'request': request_admin}))
        )
        self.assertFalse(
            permissions.is_staff_and_superuser(Context({'request': request_user}))
        )
        request_staff.user.is_superuser = True
        request_admin.user.is_staff = True
        self.assertTrue(
            permissions.is_staff_and_superuser(Context({'request': request_staff}))
        )
        self.assertTrue(
            permissions.is_staff_and_superuser(Context({'request': request_admin}))
        )
        self.assertFalse(
            permissions.is_staff_and_superuser(Context({'request': request_user}))
        )

    def test_request_objects(self):
        request_user = self.get_request(user=self.user)
        request_admin = self.get_request()
        self.assertFalse(
            permissions.is_public(Context({'request': request_user}))
        )
        self.assertFalse(
            permissions.is_public(Context({'request': request_admin}))
        )
        self.assertFalse(
            permissions.is_authenticated(Context({'request': request_admin}))
        )
        self.user.is_public = True
        self.user.save()
        self.assertTrue(
            permissions.is_public(Context({'request': request_user}))
        )
        self.assertFalse(
            permissions.is_public(Context({'request': request_admin}))
        )

    def test_negation_request_objects(self):
        request_user = self.get_request(user=self.user)
        request_admin = self.get_request(user=self.admin)
        self.assertTrue(
            permissions.is_authenticated(Context({'request': request_user}))
        )
        self.assertTrue(
            permissions.is_authenticated(Context({'request': request_admin}))
        )
        self.assertFalse(
            permissions.is_public(Context({'request': request_user}))
        )
        self.assertFalse(
            permissions.is_public(Context({'request': request_admin}))
        )
        self.user.is_public = True
        self.user.save()
        self.admin.is_public = True
        self.admin.save()
        self.assertTrue(
            permissions.is_public(Context({'request': request_user}))
        )
        self.assertTrue(
            permissions.is_public(Context({'request': request_admin}))
        )
        self.assertFalse(
            permissions.is_not_public_and_is_not_authenticated(Context({'request': request_user}))
        )
        self.assertFalse(
            permissions.is_not_public_and_is_not_authenticated(Context({'request': request_admin}))
        )

    def test_request_object_ownership(self):
        request = self.get_request(user=self.owner)
        request.object = TestObject.objects.create(title='Test!', owner=self.owner)
        self.assertTrue(
            permissions.is_owner(Context({'request': request}))
        )
        request.user = self.staff
        self.assertFalse(
            permissions.is_owner(Context({'request': request}))
        )
        self.assertTrue(
            permissions.is_owner_or_staff(Context({'request': request}))
        )
        request.user = self.admin
        self.assertFalse(
            permissions.is_owner_or_staff(Context({'request': request}))
        )


class PermissionsDjangoViewsTestCase(UtilityTestCase):

    def setUp(self):
        super().setUp()
        self.test_object = TestObject.objects.create(title='Test!', owner=self.owner)

    def test_authenticated(self):
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(response, 'Login')
        self.login(self.client, 'user')
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(response, 'Passed')

    @mock.patch('permissionsx.settings.LOGOUT_IF_DENIED', True)
    def test_settings_logout(self):
        self.login(self.client, 'user')
        self.assertTrue(auth.get_user(self.client).is_authenticated())
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(response, 'Passed')
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(response, 'Passed')
        response = self.client.get(reverse('superuser'), follow=True)
        self.assertFalse(auth.get_user(self.client).is_authenticated())

    @mock.patch('permissionsx.settings.REDIRECT_URL_NAME', 'login2')
    def test_settings_redirect_url(self):
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(response, 'Login2')

    def test_no_redirect_if_authenticated(self):
        request = self.get_request()
        redirect_url = RedirectView(
            request=request,
            redirect_url='/redirected/',
        ).get_redirect_url()
        self.assertEqual(redirect_url, '/redirected/?next=/')
        request.user = self.user
        redirect_url = RedirectView(
            request=request,
            redirect_url='/redirected/',
        ).get_redirect_url()
        self.assertEqual(redirect_url, '/redirected/')

    def test_response_class(self):
        response = self.client.get(reverse('response_class'), follow=True)
        self.assertContains(response, 'Access Denied')
        self.assertContains(response, 'Login')

    def test_template_tag(self):
        response = self.client.get(reverse('menu'), follow=True)
        self.assertContains(response, 'Login')
        self.login(self.client, 'user')
        response = self.client.get(reverse('menu'), follow=True)
        self.assertContains(response, 'User Menu')
        self.login(self.client, 'staff')
        response = self.client.get(reverse('menu'), follow=True)
        self.assertContains(response, 'Staff Menu')

    def test_template_tag_permissions_for_user_attrs_anonymous(self):
        self.user.username = 'user_username'
        context = {'request': self.get_request()}
        self.assertFalse(
            permissions_tag(
                context,
                'permissionsx.tests.permissions.username_eq_str'
            )
        )

    def test_template_tag_permissions_for_user_attrs_no_request(self):
        self.user.username = 'user_username'
        request = self.get_request()
        self.assertFalse(
            permissions_tag(
                {'request': request},
                'permissionsx.tests.permissions.username_eq_str'
            )
        )

    def test_template_tag_permissions_for_user_attrs_authenticated(self):
        request = self.get_request(user=self.user)
        self.assertFalse(
            permissions_tag(
                {'request': request},
                'permissionsx.tests.permissions.username_eq_str'
            )
        )
        self.user.username = 'user_username'
        request = self.get_request(user=self.user)
        self.assertTrue(
            permissions_tag(
                {'request': request},
                'permissionsx.tests.permissions.username_eq_str'
            )
        )


class DjangoDebugToolbarIntegrationTestCase(UtilityTestCase):

    @override_settings(SHOW_TOOLBAR=True)
    def test_django_debug_toolbar_rendered(self):
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(response, 'permissionsx.tests.views.LoginView')
        self.login(self.client, 'user')
        response = self.client.get(reverse('authenticated'), follow=True)
        self.assertContains(
            response,
            'permissionsx.tests.permissions.is_authenticated'
        )
        self.assertContains(
            response,
            'permissions.is_authenticated'
        )


class DjangoTastypieIntegrationTestCase(UtilityTestCase):

    def setUp(self):
        super().setUp()
        self.test_object = TestObject.objects.create(title='Test!', owner=self.owner)

    def test_tastypie_authorization_general(self):
        response = self.client.get('/api/v1/testsuperuser/1/')
        self.assertEqual(response.status_code, 401)
        self.login(self.client, 'user')
        response = self.client.get('/api/v1/testsuperuser/1/')
        self.assertEqual(response.status_code, 401)
        self.login(self.client, 'admin')
        response = self.client.get('/api/v1/testsuperuser/1/')
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Test!')

    def test_tastypie_authorization_overrides(self):
        json_data = json.dumps({'title': 'Changed!'})
        response = self.client.put(
            '/api/v1/testoverride/1/',
            json_data,
            content_type='application/json'
        )
        self.assertEqual(response.status_code, 401)
        self.login(self.client, 'user')
        response = self.client.put(
            '/api/v1/testoverride/1/',
            json_data,
            content_type='application/json'
        )
        self.assertEqual(response.status_code, 204)
        self.assertEqual(TestObject.objects.get(id=1).title, 'Changed!')
